! Main solver routines for an inefficient heat equation solver.
module core_gpu_good
    use heat

    contains

    ! Compute one time step of temperature evolution.
    ! Arguments:
    !   curr    (type(field)):  Current temperature values.
    !   prev    (type(field)):  Values from previous time step.
    !   a       (real(dp)):     Update equation constant.
    !   dt      (real(dp)):     Time step value.
    subroutine evolve(cur_nx, cur_ny, cur_dx, cur_dy, prev_data, cur_data, a, dt)
        implicit none

        integer, intent(in) :: cur_nx, cur_ny
        real(dp), intent(in) :: cur_dx, cur_dy
        real(dp), dimension(:,:), allocatable, intent(in) :: prev_data
        real(dp), dimension(:,:), allocatable, intent(out) :: cur_data   
        real(dp), intent(in) :: a, dt

        integer :: i, j

        !$omp target teams distribute collapse(2)
        do j = 1, cur_ny
            do i = 1, cur_nx
                cur_data(i, j) = prev_data(i, j) + a * dt * &
                    & ((prev_data(i-1, j) - 2.0 * prev_data(i, j) + &
                    &   prev_data(i+1, j)) / cur_dx**2 + &
                    &  (prev_data(i, j-1) - 2.0 * prev_data(i, j) + &
                    &   prev_data(i, j+1)) / cur_dy**2)
            end do
        end do
        !$omp end target teams distribute
    end subroutine evolve

end module core_gpu_good
